function [A_r_r, A_theta_theta, A_phi_phi, A_r_theta, A_r_phi, A_theta_phi, A_theta_r, A_phi_r, A_phi_theta] = design_GSHA_FullTensor(r, theta, phi, N, varargin);
%
% Calculates the gradient design matrices A_i_j that connect the vector 
% of (Schmidt-normalized) spherical harmonic expansion coefficients, 
% x = (g_1^0; g_1^1; h_1^1; g_2^0; g_2^1; h_2^1; ... g_N^N; h_N^N) 
% and the magnetic gradient components B_i_j, where "i, j" is "r", "theta" or "phi":
%        B_i_j = A_i_j*x
% Input: r(:)      radius vector (in units of the reference radius a)
%        theta(:)  colatitude    (in radians)
%        phi(:)    longitude     (in radians)
%        N         maximum degree/order
%
% [A_r, A_theta, A_phi] = design_GSHA(r, theta, phi, N, i_e_flag)
% with i_e_flag = 'int' for internal sources (g_n^m and h_n^m)
%                 'ext' for external sources (q_n^m and s_n^m) 
%
% Uses MEX file design_SHA_m if available, and Matlab program design_SHA_matlab.m else

% Modified by Stavros Kotsiaros based on design_SHA: January 2003, Nils Olsen, DSRI

% determine size of input arrays
max_size = max([size(r); size(theta); size(phi)]); 
max_length = max_size(1)*max_size(2);
% convert to matrix if input parameter is scalar
if length(r)     == 1; r = r*ones(max_size); end
if length(theta) == 1; theta = theta*ones(max_size); end
if length(phi)   == 1; phi = phi*ones(max_size); end
% check for equal length of all input parameters
if size(r) ~= size(theta) | size(r) ~= size(phi)
    error('Variables must be of equal size (or scalars)');
    return
end
% convert to row vector
r = reshape(r, max_length, 1);
theta = reshape(theta, max_length, 1);
phi = reshape(phi, max_length, 1);

if nargin == 4 
   i_e_flag = 'int'; % internal sources by default
elseif nargin > 4
   i_e_flag = varargin{1};
else
   error('At least 4 inputs required')
end 

% if exist('design_SHA_m') == 3
%     [A_r_r, A_theta_theta, A_phi_phi, A_r_theta, A_r_phi, A_theta_phi] = design_GSHA_m(r, theta, phi, N, i_e_flag);
% else
    [A_r_r, A_theta_theta, A_phi_phi, A_r_theta, A_r_phi, A_theta_phi, A_theta_r, A_phi_r, A_phi_theta] = design_GSHA_matlab(r, theta, phi, N, i_e_flag);
% end

% -----------------------------------------------------------------------

function [A_r_r, A_theta_theta, A_phi_phi, A_r_theta, A_r_phi, A_theta_phi, A_theta_r, A_phi_r, A_phi_theta] = design_GSHA_matlab(r, theta, phi, N, varargin);
% 
%
% Calculates the gradient design matrices A_i_j that connect the vector 
% of (Schmidt-normalized) spherical harmonic expansion coefficients, 
% x = (g_1^0; g_1^1; h_1^1; g_2^0; g_2^1; h_2^1; ... g_N^N; h_N^N) 
% and the magnetic gradient components B_i_j, where "i, j" is "r", "theta" or "phi":
%        B_i_j = A_i_j*x
% Input: r(:)      radius vector (in units of the reference radius a)
%        theta(:)  colatitude    (in radians)
%        phi(:)    longitude     (in radians)
%        N         maximum degree/order
%
% [A_r, A_theta, A_phi] = design_GSHA(r, theta, phi, N, i_e_flag)
% with i_e_flag = 'int' for internal sources (g_n^m and h_n^m)
%                 'ext' for external sources (q_n^m and s_n^m) 
%
% Uses MEX file design_SHA_m if available, and Matlab program design_SHA_matlab.m else

% Modified by Stavros Kotsiaros based on design_SHA: March 2001, Nils Olsen, DSRI

if nargin == 4
   i_e_flag = 'int'; % internal sources by default
elseif nargin > 4
   i_e_flag = varargin{1};
else
   error('At least 4 inputs required')
end 
a = 6371.2; %Earth Radius
N_koeff=(N+1)^2-1;

cos_theta = cos(theta);
sin_theta = sin(theta);
N_data    = length(theta);

A_r_r = zeros(N_data, N_koeff);
A_theta_theta = zeros(N_data, N_koeff);
A_phi_phi = zeros(N_data, N_koeff);
A_r_theta = zeros(N_data, N_koeff);
A_r_phi = zeros(N_data, N_koeff);
A_theta_phi = zeros(N_data, N_koeff);

A_theta_r = zeros(N_data, N_koeff);
A_phi_r = zeros(N_data, N_koeff);
A_phi_theta = zeros(N_data, N_koeff);

k=0;
for n = 1:N
   if strcmp(i_e_flag, 'int')
      r_n       = r.^(-(n+3));
   elseif strcmp(i_e_flag, 'ext')
      r_n       = r.^(n-2);
   else
      warning 'i_e_flag neither "int" nor "ext". Assumed "int".'
      r_n       = r.^(-(n+3));
   end
   
   Pnm = legendre(n, cos_theta, 'sch')';      % P_n^m and derivatives vrt. theta
   dPnm(:,n+1) =  (sqrt(n/2).*Pnm(:,n));      % first derivative for m=n
   dPnm(:,1) = -sqrt(n*(n+1)/2).*Pnm(:,2);   % first derivative for m=0
   if n > 1
       dPnm(:,2)=(sqrt(2*(n+1)*n).*Pnm(:,1)-sqrt((n+2)*(n-1)).*Pnm(:,3))./2; % first derivative for m=1
   end
   
   for m = 2:n-1                              % m=2...n-1
       dPnm(:,m+1)=(sqrt((n+m)*(n-m+1)).*Pnm(:,m)-sqrt((n+m+1)*(n-m)).*Pnm(:,m+2))./2;
   end
   if n == 1 dPnm(:,2) = sqrt(2)*dPnm(:,2); end
  
   ddPnm(:,n+1) =  (sqrt(n/2).*dPnm(:,n));      % second derivative for m=n
   ddPnm(:,1) =  -sqrt((n+1).*n/2).*dPnm(:,2); % second derivative for m=0
   if n > 1 
       ddPnm(:,2)=(sqrt(2.*(n+1).*n).*dPnm(:,1)-sqrt((n+2).*(n-1)).*dPnm(:,3))./2; % second derivative for m=1
   else
       ddPnm(:,2)=(sqrt(2.*(n+1).*n).*dPnm(:,1))./2; % second derivative for n=1, m=1     
   end

   for m = 2:n-1                              % m=2...n-1
       ddPnm(:,m+1)=(sqrt((n+m)*(n-m+1)).*dPnm(:,m)-sqrt((n+m+1)*(n-m)).*dPnm(:,m+2))./2;
   end
   
   if ~strcmp(i_e_flag, 'ext') % internal sources by default
      for m = 0:n  
         cos_phi   = cos(m*phi);
         sin_phi   = sin(m*phi);
            
         Pnm_r_r         = -(n+1)*(n+2).*Pnm(:,m+1);
         Pnm_theta_theta = (n+1).*Pnm(:,m+1)-ddPnm(:,m+1);
         Pnm_phi_phi     =((m^2./(sin_theta).^2)+(n+1)).*Pnm(:,m+1) - (cos_theta./sin_theta).*dPnm(:,m+1);
         Pnm_r_theta     = (n+2).*dPnm(:,m+1);
         Pnm_r_phi       = -((m.*(n+2))./(sin_theta)).*Pnm(:,m+1);
         Pnm_theta_phi   = -((m.*cos_theta)./(sin_theta.^2)).*Pnm(:,m+1) + (m./sin_theta).*dPnm(:,m+1);
         
         Pnm_theta_r     = (n+2).*dPnm(:,m+1);
         Pnm_phi_r       = -((m.*(n+2))./(sin_theta)).*Pnm(:,m+1);
         Pnm_phi_theta   = -((m.*cos_theta)./(sin_theta.^2)).*Pnm(:,m+1)+(m./sin_theta).*dPnm(:,m+1);
                 
         if m == 0
            k = k+1;       % terms corresponding to g_n^0
            A_r_r(:,k)         = (1/a).*r_n(:).*Pnm_r_r;
            A_theta_theta(:,k) = (1/a).*r_n(:).*Pnm_theta_theta;
            A_phi_phi(:,k)     =  (1/a).*r_n(:).*Pnm_phi_phi;
            A_r_theta(:,k)     = -(1/a).*r_n(:).*Pnm_r_theta;
            A_r_phi(:,k)       = -(1/a).*r_n(:).*Pnm_r_phi; % this correspondt to term h_n^0
            A_theta_phi(:,k)   = -(1/a).*r_n(:).*Pnm_theta_phi; % this correspondt to term h_n^0
            
            A_theta_r(:,k)     = -(1/a).*r_n(:).*Pnm_theta_r;
            A_phi_r(:,k)       = -(1/a).*r_n(:).*Pnm_phi_r;
            A_phi_theta(:,k)   = -(1/a).*r_n(:).*Pnm_phi_theta;           

         else
            k = k+1;       % terms corresponding to g_n^m
            A_r_r(:,k)         = (1/a).*r_n(:).*cos_phi.*Pnm_r_r;
            A_theta_theta(:,k) = (1/a).*r_n(:).*cos_phi.*Pnm_theta_theta;            
            A_phi_phi(:,k)     = (1/a).*r_n(:).*cos_phi.*Pnm_phi_phi;
            A_r_theta(:,k)     = -(1/a).*r_n(:).*cos_phi.*Pnm_r_theta;
            A_r_phi(:,k)       = (1/a).*r_n(:).*sin_phi.*Pnm_r_phi; 
            A_theta_phi(:,k)   = (1/a).*r_n(:).*sin_phi.*Pnm_theta_phi; 
           
            A_theta_r(:,k)     = -(1/a).*r_n(:).*cos_phi.*Pnm_theta_r;
            A_phi_r(:,k)       = (1/a).*r_n(:).*sin_phi.*Pnm_phi_r;
            A_phi_theta(:,k)   = (1/a).*r_n(:).*sin_phi.*Pnm_phi_theta;               
            
            k = k+1;       % terms corresponding to h_n^m
            A_r_r(:,k)         = (1/a).*r_n(:).*sin_phi.*Pnm_r_r;
            A_theta_theta(:,k) = (1/a).*r_n(:).*sin_phi.*Pnm_theta_theta;            
            A_phi_phi(:,k)     = (1/a).*r_n(:).*sin_phi.*Pnm_phi_phi;
            A_r_theta(:,k)     = -(1/a).*r_n(:).*sin_phi.*Pnm_r_theta;
            A_r_phi(:,k)       = -(1/a).*r_n(:).*cos_phi.*Pnm_r_phi;
            A_theta_phi(:,k)   = -(1/a).*r_n(:).*cos_phi.*Pnm_theta_phi;
             
            A_theta_r(:,k)     = -(1/a).*r_n(:).*sin_phi.*Pnm_theta_r;
            A_phi_r(:,k)       = -(1/a).*r_n(:).*cos_phi.*Pnm_phi_r;
            A_phi_theta(:,k)   = -(1/a).*r_n(:).*cos_phi.*Pnm_phi_theta;               
         end   
%       test(:,k) = (Pnm_phi_phi+Pnm_theta_theta-Pnm_r_r);
      end % m
       
   else % external sources, i_e_flag ='ext'
      for m = 0:n  
         cos_phi   = cos(m*phi);
         sin_phi   = sin(m*phi);
         
         Pnm_r_r = -n*(n-1).*Pnm(:,m+1);
         Pnm_theta_theta= -n.*Pnm(:,m+1)-ddPnm(:,m+1);
         Pnm_phi_phi =((m^2./(sin_theta).^2)-n).*Pnm(:,m+1) - (cos_theta./sin_theta).*dPnm(:,m+1);
         Pnm_r_theta = -(n-1).*dPnm(:,m+1);
         Pnm_r_phi = ((m.*(n-1))./(sin_theta)).*Pnm(:,m+1);
         Pnm_theta_phi = -((m.*cos_theta)./(sin_theta.^2)).*Pnm(:,m+1) + (m./sin_theta).*dPnm(:,m+1);
         
         
         
        if m == 0
            k = k+1;       % terms corresponding to g_n^0
            A_r_r(:,k) = (1/a).*r_n(:).*Pnm_r_r;
            A_theta_theta(:,k) = (1/a).*r_n(:).*Pnm_theta_theta;
            A_phi_phi(:,k) =  (1/a).*r_n(:).*Pnm_phi_phi;
            A_r_theta(:,k) = (1/a).*r_n(:).*Pnm_r_theta;
            A_r_phi(:,k) = -(1/a).*r_n(:).*Pnm_r_phi; % this correspondt to term h_n^0
            A_theta_phi(:,k) = -(1/a).*r_n(:).*Pnm_theta_phi; % this correspondt to term h_n^0
            
            

         else
            k = k+1;       % terms corresponding to g_n^m
            A_r_r(:,k) = (1/a).*r_n(:).*cos_phi.*Pnm_r_r;
            A_theta_theta(:,k) = (1/a).*r_n(:).*cos_phi.*Pnm_theta_theta;            
            A_phi_phi(:,k) = (1/a).*r_n(:).*cos_phi.*Pnm_phi_phi;
            A_r_theta(:,k) = (1/a).*r_n(:).*cos_phi.*Pnm_r_theta;
            A_r_phi(:,k) = (1/a).*r_n(:).*sin_phi.*Pnm_r_phi;
            A_theta_phi(:,k) = (1/a).*r_n(:).*sin_phi.*Pnm_theta_phi; 
             
            
            
            k = k+1;       % terms corresponding to h_n^m
            A_r_r(:,k) = (1/a).*r_n(:).*sin_phi.*Pnm_r_r;
            A_theta_theta(:,k) = (1/a).*r_n(:).*sin_phi.*Pnm_theta_theta;            
            A_phi_phi(:,k)=(1/a).*r_n(:).*sin_phi.*Pnm_phi_phi;
            A_r_theta(:,k) = (1/a).*r_n(:).*sin_phi.*Pnm_r_theta;
            A_r_phi(:,k) = -(1/a).*r_n(:).*cos_phi.*Pnm_r_phi;
            A_theta_phi(:,k) = -(1/a).*r_n(:).*cos_phi.*Pnm_theta_phi;
             
            
         end   
      end % m
   end % if int/ext
end % n
% plot(test)

